/*
 * jSite2 - Main.java -
 * Copyright © 2008 David Roden
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package net.pterodactylus.jsite.main;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.UIManager;
import javax.swing.UIManager.LookAndFeelInfo;

import net.pterodactylus.jsite.core.CoreImpl;
import net.pterodactylus.jsite.core.NodeManager;
import net.pterodactylus.jsite.core.ProjectManager;
import net.pterodactylus.jsite.core.RequestManager;
import net.pterodactylus.jsite.gui.SwingInterface;
import net.pterodactylus.util.logging.Logging;

/**
 * Main class that is called by the VM.
 *
 * @author David ‘Bombe’ Roden &lt;bombe@freenetproject.org&gt;
 */
public class Main {

	/**
	 * Main entry method for the VM.
	 *
	 * @param args
	 *            The command-line arguments
	 */
	public static void main(String[] args) {
		new Main().start();
	}

	/**
	 * Starts the core and the default {@link SwingInterface}.
	 */
	private void start() {
		Logging.setup("jSite");

		CoreImpl core = new CoreImpl();

		String configDirectory = System.getProperty("user.home") + File.separator + ".jSite";

		NodeManager nodeManager = new NodeManager("jSite-" + Version.getVersion(), configDirectory);
		core.setNodeManager(nodeManager);
		nodeManager.addNodeListener(core);

		ProjectManager projectManager = new ProjectManager(configDirectory);
		core.setProjectManager(projectManager);
		projectManager.setNodeManager(nodeManager);

		RequestManager requestManager = new RequestManager();
		core.setRequestManager(requestManager);
		nodeManager.addNodeListener(requestManager);
		requestManager.setNodeManager(nodeManager);
		requestManager.addRequestListener(core);

		SwingInterface swingInterface = new SwingInterface(core, configDirectory);
		core.addCoreListener(swingInterface);
		Logging.addLoggingListener(swingInterface);

		core.start();
	}

	/**
	 * Tries to load the class with the given name and includes the look & feel
	 * in the UIManager, if it exists.
	 *
	 * @param name
	 *            The name of the look & feel
	 * @param className
	 *            The name of the look & feel’s main class
	 */
	@SuppressWarnings("unused")
	private void addLookAndFeel(String name, String className) {
		addLookAndFeels(new LookAndFeelInfo(name, className));
	}

	/**
	 * Tries to load each look & feel and adds it to the list of installed look &
	 * feels.
	 *
	 * @see UIManager#setInstalledLookAndFeels(LookAndFeelInfo[])
	 * @param lookAndFeelInfos
	 *            The look & feels to add
	 */
	private void addLookAndFeels(LookAndFeelInfo... lookAndFeelInfos) {
		List<LookAndFeelInfo> allLookAndFeelInfos = new ArrayList<LookAndFeelInfo>(Arrays.asList(UIManager.getInstalledLookAndFeels()));
		for (LookAndFeelInfo lookAndFeelInfo : lookAndFeelInfos) {
			try {
				Class.forName(lookAndFeelInfo.getClassName());
				allLookAndFeelInfos.add(lookAndFeelInfo);
			} catch (ClassNotFoundException e) {
				/* okay, it doesn't exist, ignore. */
			}
		}
		UIManager.setInstalledLookAndFeels(allLookAndFeelInfos.toArray(new LookAndFeelInfo[0]));
	}

}
